/*
 * Copyright (C) 2005-2007 Alfresco Software Limited.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

 * As a special exception to the terms and conditions of version 2.0 of 
 * the GPL, you may redistribute this Program in connection with Free/Libre 
 * and Open Source Software ("FLOSS") applications as described in Alfresco's 
 * FLOSS exception.  You should have recieved a copy of the text describing 
 * the FLOSS exception, and it is also available here: 
 * http://www.alfresco.com/legal/licensing
 *
 * @author Alfresco
 * @author Laurent Meunier <lme@atolcd.com>
 */
package com.atolcd.alfresco;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.alfresco.connector.User;
import org.alfresco.web.site.AuthenticationUtil;
import org.alfresco.web.site.FrameworkHelper;
import org.alfresco.web.site.RequestContext;
import org.alfresco.web.site.RequestUtil;
import org.alfresco.web.site.UserFactory;
import org.alfresco.web.site.exception.RequestContextException;
import org.jasig.cas.client.util.AbstractCasFilter;
import org.jasig.cas.client.validation.Assertion;


/**
 * Share - CAS authentication
 * 
 */
public class CasAuthenticationFilter implements Filter {

	public CasAuthenticationFilter() {
		super();
	}
	public void destroy() {
	}
	public void init(FilterConfig config) throws ServletException {
	}

	/**
	 * Run the filter
	 * 
	 * @param sreq
	 *            ServletRequest
	 * @param sresp
	 *            ServletResponse
	 * @param chain
	 *            FilterChain
	 * @exception IOException
	 * @exception ServletException
	 * @throws  
	 */
	public void doFilter(ServletRequest sreq, ServletResponse sresp, FilterChain chain) throws IOException, ServletException {
		// Get the HTTP request/response/session
		HttpServletRequest req = (HttpServletRequest) sreq;
		HttpServletResponse resp = (HttpServletResponse) sresp;
		HttpSession httpSess = req.getSession(true);

		// check if user is already authenticated
		try {
			RequestContext context = RequestUtil.getRequestContext(req);
			User user = context.getUser();
			if(user != null && !user.getId().equals(UserFactory.USER_GUEST)) {
				// already authenticated
				chain.doFilter(sreq, sresp);
				return;
			}
		} catch (RequestContextException e) {
			e.printStackTrace();
		}

		// get CAS informations
		Assertion assertion = (Assertion) httpSess.getAttribute(AbstractCasFilter.CONST_CAS_ASSERTION);
		String username = assertion.getPrincipal().getName();
		String proxyticket = assertion.getPrincipal().getProxyTicketFor("http://ged.atolcd.priv/alfresco");
		
		try {
			// pass the proxy CAS ticket to alfresco to authenticate (and get an alfresco ticket)
			UserFactory userFactory = FrameworkHelper.getUserFactory();
			boolean authenticated = userFactory.authenticate(req, username, proxyticket);
			if (authenticated) {
				// this will fully reset all connector sessions
				AuthenticationUtil.login(req, resp, username);
			}
		} catch (Throwable err) {
			throw new ServletException(err);
		}

		chain.doFilter(sreq, sresp);
	}
}
